# Runs our pixel test against all the files in the given folder.
# Certain parts have been commented out due to limitations on running all at once
import moo_nsde
import soo_de

import os
import sys
import shutil

from keras.applications import vgg16
from keras.applications import vgg19
from keras.applications import resnet

output_dir = "output_data"
argv = len(sys.argv)
print(argv)
file = sys.argv[1] if argv > 1 else "images_to_run/ILSVRC2012_val_00046372.JPEG"
start = sys.argv[2] if argv > 2 else 1
algorithm = sys.argv[3] if argv > 3 else "dea"
model_type = sys.argv[4] if argv > 4 else "vgg16"

print(file)
print(start)
print(algorithm)
print(model_type)

match model_type:
    case "vgg16":
        model = vgg16.VGG16(weights='imagenet')
    case "vgg19":
        model = vgg19.VGG19(weights='imagenet')
    case "r50":
        model = resnet.ResNet50(weights='imagenet')
    case "r101":
        model = resnet.ResNet101(weights='imagenet')
    case "r152":
        model = resnet.ResNet152(weights='imagenet')
    case _:
        print(f"Unknown model option: vgg16, vgg19, r50, r101, r152")

# Split the dir down to be just the file name
# file_name = file.split("/")[1].split('.')[0]
file_name = file.split("/")[1].split('.')[0]
print(file_name)

# Create the new output dir if it doesn't exist
if not os.path.isdir(f"{output_dir}/{algorithm}_{model_type}_{file_name}"):
    os.mkdir(f"{output_dir}/{algorithm}_{model_type}_{file_name}")

for iteration in range(int(1), 2):
    path_name = f"{algorithm}_{model_type}_{file_name}/{iteration}"
    # If the output directory exists then remove it for the new results
    if os.path.isdir(f"{output_dir}/{path_name}"):
        print(path_name, 'exists in the destination path!')
        shutil.rmtree(f"{output_dir}/{path_name}")
    os.mkdir(f"{output_dir}/{path_name}")
    print(path_name)
    print(f"Running algorithm {algorithm} with model {model_type}{model}")
    match algorithm:
        case "dea":
            soo_de.run_mutations(20, 400, file, model, f'{output_dir}/{path_name}')
        case "nsde":
            moo_nsde.run_mutations(20, 250, file, model, f'{output_dir}/{path_name}')
        case _:
            print("algorithm type not recognised .. (dea, nsde)")
