# Python script to get random images from the imagenet test images
# Get a random image and run it through the vgg16 model to get a baseline prediction
# Only keep images with +.8% accuracy

import os
import random
import json
import scipy.io

import PIL.Image
import numpy as np
from keras.applications import vgg19
from keras.applications import vgg16
from keras.utils.image_utils import load_img, img_to_array

from keras.applications.imagenet_utils import preprocess_input, decode_predictions


def get_predictions(model, image_array):
    image_batch = np.expand_dims(image_array, axis=0)
    processed = preprocess_input(image_batch)
    predictions = model.predict(processed, verbose=0)
    return decode_predictions(predictions, top=10)[0]


def get_images_in_folder(folder):
    list_of_images = []
    for root, dirs, files in os.walk(folder):
        for file in files:
            list_of_images.append(os.path.join(root, file))
    return list_of_images


def validate_ground_truth(image):
    image = image.split('/')[1].split('_')[2].split('.')[0]
    ground_truth = truth_data[int(image) - 1]
    return(synsets[int(ground_truth) -1][0][1][0])

vgg16model = vgg16.VGG16(weights='imagenet')
images_to_run = {}
images_to_replace = []
synsets = scipy.io.loadmat('generate_images/meta_data/meta.mat')['synsets']
with open('generate_images/meta_data/ILSVRC2012_validation_ground_truth.txt') as f:
    truth_data = f.readlines()

for image in get_images_in_folder("images_to_run/"):
    original_image = load_img(image)
    resized = original_image.resize([224, 224], PIL.Image.BILINEAR)
    image_as_array = img_to_array(resized)
    original_classification = get_predictions(vgg16model, image_as_array)
    if original_classification[0][0] in images_to_run:
        images_to_replace.append(image)
    elif (0.8 < original_classification[0][2] < 0.9) and (original_classification[0][0] == validate_ground_truth(image)):
        images_to_run[original_classification[0][0]] = {"image": image, "prediction": str(original_classification[0][1]), "confidence": str(original_classification[0][2])}
    else:
        images_to_replace.append(image)

print(f"total to replace: {len(images_to_replace)}, {images_to_replace}")
print(f"images in array: {len(images_to_run)}")

for image in images_to_replace:
    os.remove(image)

with open(f"generate_images/images_to_use.json", 'w') as convert_file:
    convert_file.write(json.dumps(images_to_run, indent=6))