import math
import numpy as np

from pymoo.algorithms.base.genetic import GeneticAlgorithm
from pymoo.core.callback import Callback


# PredictionsAndImage - Used for storing information after each evaluation.
# self.data['*'] - Adds each prediction item as a key value with each returned value from _evaluate.
# self.data['image'] - Stores the most recent image details.
class PredictionsAndImage(Callback):
    def __init__(self) -> None:
        super().__init__()
        self.data['image'] = []
        self.data['matrix'] = []
        self.data['flipped'] = False
        self.data['flip_matrix'] = None
        self.data['predictions'] = {}
        self.data['first_rank'] = []
        self.data['kill_it'] = 5


    def notify(self, algorithm: GeneticAlgorithm):
        """
        :type algorithm: object
        """
        # let's focus only on non-dominated solutions (front zero)
        pop: object
        g = []
        for individual in algorithm.pop:
            if 'rank' in individual.data:
                if individual.data['rank'] == 0:
                    found = False
                    if len(self.data['first_rank']) <= 0:
                        found = False
                    for ranked in self.data['first_rank']:
                        if ranked['f'][0] == individual.F[0] and ranked['f'][1] == individual.F[1]:
                            found = True
                    if not found:
                        g = individual.get("G")
                        first = {'f': individual.F, 'matrix': g[len(g)-2], 'n_gen': algorithm.n_gen}
                        self.data['first_rank'].append(first)
            else:
                break

        # let's consider the solution on the front with most variations on the predictions
        # aka the extreme of the non-dominated front
        best_f1 = math.inf
        g = []
        for individual in algorithm.pop:
            if individual.F[0] < best_f1:
                best_f1 = individual.F[0]
                g = individual.get("G")

        self.data['image'] = g[len(g)-3]
        self.data['matrix'] = g[len(g)-2]

        if len(g) > 0:
            print(f"flipped? {g[len(g) - 1]}")
            if not self.data['flipped'] and g[len(g)-1]:
                self.data['flipped'] = True
                self.data['flip_matrix'] = self.data['matrix']
                self.data['n_gen'] = algorithm.n_gen

            g = np.delete(g, (len(g)-1))

            for prediction in g:
                if len(prediction) > 1:
                    if isinstance(prediction[1], str):
                        if prediction[1] in self.data['predictions']:
                            self.data['predictions'][prediction[1]].append(float(prediction[2]))
                        else:
                            self.data['predictions'][prediction[1]] = []
                            self.data['predictions'][prediction[1]].append(float(prediction[2]))

        # If we've flipped our prediction then give 5 extra runs to ensure there's no outliers we might miss
        # Enable when testing so things end quicker
        if self.data['flipped']:
            if self.data['kill_it'] >= 0:
                self.data['kill_it'] -= 1
            else:
                algorithm.termination.force_termination = True
