import json
from src import visualisations


def output_json(data, file_name):
    with open(f"{file_name}/data.json", 'w') as convert_file:
        convert_file.write(json.dumps(data, indent=6))


def get_pareto_from_data(data):
    best_pareto = []
    best_fitness = 1.0
    for f in data.F:
        for pareto in data.algorithm.callback.data['first_rank']:
            if f[0] == pareto['f'][0]:
                if abs(best_fitness) > abs(pareto['f'][0]) > 0:
                    # Store our best fitness along with the amount of changes
                    best_fitness = pareto['f'][0]
                    result = f.tolist()
                    result.append(convert_matrix_format(pareto['matrix']))
                    result.append(pareto['n_gen'])
                    best_pareto = result
    print(best_pareto)
    return best_pareto


# Helper for converting the matrix array due to its use of np arrays
# These need modifying so they can be stored in the json file.
def convert_matrix_format(matrix_array):
    if matrix_array is None:
        return None
    new_matrix = []
    for m in matrix_array:
        new_matrix.append([int(m[0]), int(m[1]), m[2].tolist(), str(m[3])])
    return new_matrix


# Wrapper method that performs all our data output operations
# Takes the original problem, the results from the test and the directory to store results
def output_data(problem, res, directory):
    # Creates a dictionary object containing all our data
    # Formats it so we can output to json

    # Create a variable containing our callback output_data.
    val = res.algorithm.callback.data
    no_image = val.copy()
    no_image.pop('image')
    no_image.pop('matrix')
    no_image.pop('first_rank')

    no_image['execution_time'] = res.exec_time

    if 'nsga2' in directory or 'nsde' in directory:
        no_image.pop('flip_matrix')
        no_image['best_pareto'] = get_pareto_from_data(res)
        if len(no_image['best_pareto']) > 0:
            no_image['matrix'] = no_image['best_pareto'][3]
            no_image['noise'] = no_image['best_pareto'][2]
            no_image['matrix_size'] = len(no_image['matrix'])
            no_image['n_gen'] = no_image['best_pareto'][4]
        no_image['seed'] = res.algorithm.seed
        no_image['pareto'] = res.F.tolist()

    if 'dea' in directory or 'gaa' in directory:
        no_image['matrix'] = convert_matrix_format(no_image['flip_matrix'])
        no_image['matrix_size'] = len(no_image['flip_matrix']) if no_image['flip_matrix'] else 0
        no_image['seed'] = res.algorithm.seed
        no_image.pop('flip_matrix')
        no_image['pareto'] = res.F.tolist()

    # Outputs the data.json file in the given directory
    output_json(no_image, directory)
    # Outputs all our visualisations
    visualisations.show_and_save_line_plot(val, directory)
    visualisations.show_and_save_scatter_plot(problem, res, directory)
    if no_image['matrix']:
        visualisations.show_and_save_final_image(res, no_image['matrix'], directory)
        visualisations.show_and_save_matrix_overlay(no_image['matrix'], directory)