from abc import ABC

import numpy as np
import PIL.Image
from pymoo.core.problem import ElementwiseProblem
from keras.applications import vgg16
from keras.utils.image_utils import load_img, img_to_array
from keras.applications.imagenet_utils import preprocess_input, decode_predictions


# Super class extending Elementwiseproblem, used by all our problems
class PixelChangeMinimise(ElementwiseProblem, ABC):
    def __init__(self,
                 image="images/red_panda.jpg",
                 model=vgg16.VGG16(weights='imagenet')):
        super().__init__(n_var=3, n_obj=1, n_ieq_constr=0)
        self.model = model
        self.original_image = resized = load_img(image).resize([224, 224], PIL.Image.BILINEAR)
        self.image_as_array = img_to_array(self.original_image, dtype=int)
        self.rows = len(self.image_as_array)
        self.cols = len(self.image_as_array[0])

        # we save the predictions done for the original picture
        self.original_classification = self.get_predictions(self.image_as_array)

    """
    get_predictions - Returns an array of the top 10 predictions.
    image_array - The image array we're getting the predictions of.
    """
    def get_predictions(self, image_array):
        image_batch = np.expand_dims(image_array, axis=0)
        processed = preprocess_input(image_batch)
        predictions = self.model.predict(processed, verbose=0)
        return decode_predictions(predictions, top=10)[0]
      