# Handle the visualisations
import numpy as np
from keras_preprocessing.image.utils import array_to_img
from pymoo.visualization.scatter import Scatter
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure


# Creates a scatter plot with the pareto fron data
# Shows and image if show=True
def show_and_save_scatter_plot(problem, res, folder, show=False):
    plot = Scatter()
    plot.reset()
    plot.add(res.F, facecolor="none", edgecolor="red")
    plot.save(f"{folder}/scatter.png")
    if show:
        plot.show()


# Creates a line plot with all our prediction data
# Shows an image at the end if show=True
def show_and_save_line_plot(val, folder, show=False):
    # Iterate through the predictions and create a new plot on our graph. Generate the legend based off of our keys.
    plt.clf()
    # figure(figsize=(12, 10), dpi=100)
    # plt.rc('font', size=28)
    # plt.rc('axes', labelsize=12)
    legend = []
    for item in val['predictions']:
        if item != 'image':
            plt.plot(np.arange(len(val['predictions'][item])), val['predictions'][item], label=item)
            legend.append(item)

    plt.xlabel('Generations')
    plt.ylabel('Prediction Fitness Value')
    ax = plt.subplot(111)
    chartBox = ax.get_position()
    ax.set_position([chartBox.x0, chartBox.y0, chartBox.width * 0.8, chartBox.height])
    ax.legend(loc='upper center', bbox_to_anchor=(1.2, 0.8), shadow=True, ncol=1)
    # plt.legend(legend, bbox_to_anchor=(1, 1), shadow=True, ncol=1, loc="upper left")
    plt.savefig(f"{folder}/line.png", bbox_inches='tight')
    if show:
        plt.show()


# Create an image of the 'final' image
# Show the final image if show=False
def show_and_save_final_image(res, matrix, folder, show=False):

    # Save the final image to file
    # plt.clf()
    seed = res.problem.image_as_array
    for point in matrix:
        seed[point[0]][point[1]] = point[2]
    img = array_to_img(seed)

    img.save(f"{folder}/final_image.jpg")
    if show:
        plt.imshow(img)
        plt.show()


# Creates an image containing the matrix overlay map
# Shows the image if show=True
def show_and_save_matrix_overlay(matrix, folder, show=False):
    img = np.zeros([224, 224, 3], dtype=np.uint8)
    img[:] = [255., 255., 255.]
    print(img)
    for point in matrix:
        img[int(point[0])][int(point[1])] = [1., 1., 1.]
    matrix_applied = array_to_img(img)
    matrix_applied.save(f"{folder}/matrix_overlay.png")
    if show:
        plt.imshow(matrix_applied)
        plt.show()
