# This document describes upcoming changes we should consider for this project.

---
### Points to consider

* Paper lacks a more comprehensive work section
* From a geometrical point of view, the selected point corresponds to the central point in the front." -> not necessarily. This only happens if the solutions are evenly distributed above and below the O1 fitness landscape. There is no indication that this happens. In fact, if the assumption that DL models are easily fooled is real, we can expect very few solutions with O1 > 0. For example, if changing 0.5% of the pixels can flip a label, the chosen solution will be much closer to an edge than the center of the front.
* **Comparison with a well known (state-of-the-art) black box test technique**  (This is mentioned a couple times)
* The problem is not well framed
* Evaluation needs improvement
* Link to the replication package is not available
* Reframe the context of the solution to be more of security rather than functional testing
* Discuss more the motivation for having a minimal amount of changes. (Less detectable in an attack)
* One pixel solution seems hacky, we need to show why our solution is better across multiple ranges. They claim good coverage of the kaggle sets but low coverage with imagenet. I think we should focus on images with high confidence on the original image to show we can disrupt with minimal changes.
* Choose images randomly from imagenet that have a high initial confidence.
* Section 4.3 - Should better describe why we run each image 10 times with a different random seed and why we retain this seed for repeat executions
* Statistical analysis - Fisher exact method for statistical analysis - https://www.simulamet.no/sites/default/files/publications/Simula.simula.670.pdf
* Consider adding the amount of generations to flip the image
* Efficiency of the approach
* The relevance of the test is called into question (reviewer 3) Why do we need black box! Are there DNNs where existing classifiers cannot be applied.
* Why should we care about the smallest amount of perturbations?!
* How one could address the problem of ensuring the altered images maintain their original semantics.
* Validating whether the selected seeds are correctly labeled.
* Include white/grey box approaches to better contextualize the relevance of this work.
* Studying this approach when applied to larger images. How results change when images contain more pixels.

### Errors in current document
* Which takes as input an image i ∈ and returns a label l ∈ L -> there is a capital I missing here.
* This operator creates to -> This operator creates two
* benchmark challenges challenges -> repeated word?
* Pg. 2: the authors call their research question RQ1. I would remove the suffix '1' since there is only one research question.
* Pg. 3: "Secondly, while [23] proposes a Discrete Cosine Transform-based method...", for readability purposes it would be better not to use the reference as part of the sentence. It would be better to rephrase "Secondly, while Suzuki et al. [23] propose a Discrete Cosine Transform-based method..."
Remaining on this sentence it is not clear to me what the authors want to say. My understanding is that the Discrete Cosine Transform-based method is more complex than a pixel-based method. Is that correct? If that is the case, complexity in the sense of computation time?
* Pg. 3: "...which takes as input an image i \in and returns a label..." -> "which takes as input an image i \in I and returns a label..."
* Pg. 4: "In this paper, we reformulate this problem as a multi-objective where both..." -> "In this paper, we reformulate this problem as a multi-objective problem where both..."
* Pg. 4, equation 2: maybe the symbol 'e_a' needs to be replaced with 'a'?
* Pg. 5: "...the problem is inherently multi-objective where the goal becomes find the set of optimal..." -> "...the problem is inherently multi-objective where the goal becomes to find the set of optimal..."
* Pg. 5: "...the chance of the most diverse solutions within the same front to be selected for the next generation Then, N solutions are selected..." -> "...the chance of the most diverse solutions within the same from to be selected for the next generation. Then, N solutions are selected..."
* Pg. 6: "This operator creates to new attacks by combining..." -> "This operator creates two new attacks by combining..."?
* Pg. 7: "...got a canonical status due to its strong performance in the ImageNet benchmark challenges challenges." -> "...got a canonical status due to its strong performance in the ImageNet benchmark challenges."
* Pg. 7: "The model takes as input images of size 224 x 224 x 3 pixels and can prediction 1000 different type of objects (or labels)." -> "The model takes as input images of size 224 x 224 x 3 and can classify 1000 different types of objects (or labels)."
* Pg. 7: "To show the effectiveness of multi object optimisation compared..." -> "To show the effectiveness of multi-objective optimisation compared...".
* Pg. 7: "Each image is chosen such, that the main..." -> "Each image is chosen such that the main..."
* Pg. 7: "We set our input parameters for both the many- and the single-objective genetic algorithms..." -> "We set our input parameters for both the multi-objective and the single-objective genetic algorithms..."
* Pg. 8: "Each picture in the dataset..." -> "Each image in the dataset..."
* Pg. 9, Table 1: The empty row for the seed image "meerkats" can be replaced with a footnote that says that for the seed image "meerkats" both algorithms failed to generate an adversarial attack. Moreover, it would help readability to add a row with the average values of success rate and number of changed pixels.
Moreover, in the "Seed Image" column the authors report the "superclass" for some images instead of the ImageNet class. For example "dog" is not an ImageNet class, rather "whippet" or "Saluki" is. I think it would be less confusing to consider the ImageNet class for every seed image.
* Pg. 10, Figure 1: first of all this figure is placed before the others in the text and commented as last. I would move it to the next page and give it a name which follows the order in which the figure is commented in the text (the comment also holds for the other images). Secondly, the figure is small and not very readable. I would try to transpose it, if it does not take much space, or alternatively I would show only the most interesting cases.
* Pg. 11, Figure 2: also this figure is small. What I would do here is to keep only the original class and the "mispredicted" class (e.g. respectively "whippet" and "English_foxhound" in Fig. 2a), such that the legend fits inside the plot frame and it would be possible to make the plot bigger. Then, I would increase the linewidth of the plot to make the lines more visible. Finally, I would try to make the text in the figures as big as possible.
* Pg. 11: "...may be degraded such, that a change in ground truth label would be needed and..." -> "...may be degraded such that a change in ground truth label would be needed and..."
* Pg. 12 Figure 3 and Figure 4: it is not easy to count the number of pixels in the mutation mask; I would write the respective numbers in the subfigure b of both figures. In the subfigures a and c the labels are a bit unclear to me. I would have wrote the label of the original image in the subfigures a and the label of the mutated image in the subfigures c. Therefore, following Figure 2a, I would write in Figure 3a the label "whippet" and in Figure 3c the label "English_foxhound". Similarly, following Figure 2b, I would write in Figure 4a the label "whippet" and in Figure 4c the label "Saluki".

## Paper for extra reading

---
* https://urldefense.com/v3/__https://link.springer.com/chapter/10.1007/978-3-030-41579-2_35__;!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6YrqRVQEZ$ 
* https://urldefense.com/v3/__https://ieeexplore.ieee.org/abstract/document/8923450__;!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6Ysn2GPo8$ 
* https://urldefense.com/v3/__https://www.sciencedirect.com/science/article/pii/S0893608020301350?via*3Dihub__;JQ!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6YpNtG67H$   (this is not MOEA, but still black-box attacks using EAs)
* https://urldefense.com/v3/__https://link.springer.com/chapter/10.1007/978-3-030-72699-7_35__;!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6Yix8gNyl$ 
* https://urldefense.com/v3/__https://dl.acm.org/doi/abs/10.1145/3128572.3140448__;!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6Yh16ADDK$ 
* https://urldefense.com/v3/__https://www.sciencedirect.com/science/article/pii/S0045790621003670__;!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6YmogBq_D$ 
* https://urldefense.com/v3/__https://ieeexplore.ieee.org/abstract/document/9760177__;!!PAKc-5URQlI!-GiIUTwwFfuJ8K1Wr5upgV58hW1PMBm7_qVgr6u_JXyKXeFhLq44sZzEhC7jj3fw-8cWGLvemgzWAXw6Yt8ho6bj$
* Riccio V, Jahangirova G, Stocco A, Humbatova N, Weiss M, Tonella P. Testing machine learning based systems: a systematic mapping. Empirical Software Engineering. 2020 Nov;25(6):5193-254.
* Su J, Vargas DV, Sakurai K. One pixel attack for fooling deep neural networks. IEEE Transactions on Evolutionary Computation. 2019 Jan 4;23(5):828-41.
* Narodytska N, Kasiviswanathan SP. Simple Black-Box Adversarial Attacks on Deep Neural Networks. In CVPR Workshops 2017 Jul 21 (Vol. 2, p. 2).
* Arcuri A, Briand L. A hitchhiker's guide to statistical tests for assessing randomized algorithms in software engineering. Software Testing, Verification and Reliability. 2014 May;24(3):219-50.